package de.duehl.gameoflife.ui;

/**
 * Dieses Interface stellt die Verbindung zwischen dem GameOfLifePanel und dem
 * GameOfLifeMouseMotionListener sowie dem GameOfLifeMouseListener dar.
 *
 * @version 1.01     2013-10-31
 * @author Christian Dühl
 */

public interface MouseWatcher {

    /**
     * Meldet einen Mausklick.
     *
     * @param mouseX
     *            x-Koordinate des Mausklicks.
     * @param mouseY
     *            y-Koordinate des Mausklicks.
     * @param buttonNumber
     *            Nummer des Mausknopfes (vgl. java.awt.event.MouseEvent)
     *            1:links, 2:mitte, 3:rechts
     *            Zur Abfrage kann MouseEvent.BUTTON1 etc. verwendet werden.
     */
    void mousePressedAt(int mouseX, int mouseY, int buttonNumber);

    /**
     * Meldet das Ziehen der Maus während eines Mausklicks.
     *
     * @param mouseX
     *            x-Koordinate des Mausklicks.
     * @param mouseY
     *            y-Koordinate des Mausklicks.
     * @param buttonNumber
     *            Nummer des Mausknopfes (vgl. java.awt.event.MouseEvent)
     *            1:links, 2:mitte, 3:rechts
     *            Zur Abfrage kann MouseEvent.BUTTON1 etc. verwendet werden.
     */
    void mouseDraggedTo(int mouseX, int mouseY, int buttonNumber);

    /**
     * Meldet das Lösen des Mausklicks.
     *
     * @param mouseX
     *            x-Koordinate des Mausklicks.
     * @param mouseY
     *            y-Koordinate des Mausklicks.
     * @param buttonNumber
     *            Nummer des Mausknopfes (vgl. java.awt.event.MouseEvent)
     *            1:links, 2:mitte, 3:rechts
     *            Zur Abfrage kann MouseEvent.BUTTON1 etc. verwendet werden.
     */
    void mouseReleasedAt(int mouseX, int mouseY, int buttonNumber);

}
