package de.duehl.gameoflife.ui.dialogs;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import de.duehl.swing.ui.colors.Colorizer;
import de.duehl.swing.ui.dialogs.base.ModalDialogBase;

/**
 * Diese Klasse stellt einen Dialog dar, der aufgerufen wird, wenn der Benutzer
 * das Fenster schließt.
 *
 * @version 1.02     2019-07-08
 * @author Christian Dühl
 */

public class QuitDialog extends ModalDialogBase {

    /** Rahmen der Oberfläche, vor der dieser Dialog erzeugt wird. */
    private JFrame parentFrame;

    /** Ergebnis des Dialogs. */
    private boolean quit;

    /**
     * Konstruktor.
     *
     * @param colorizer
     *            Farbverwaltung für die Gui.
     * @param parentFrame
     *            Rahmen der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Bild des Programms.
     */
    public QuitDialog(Colorizer colorizer, JFrame parentFrame, Image programImage) {
        super(parentFrame.getLocation(), programImage, "Game of Life beenden?", colorizer);
        setColors(getWindowAsComponent());
        //setPreferredSize(new Dimension(550, 600));

        this.parentFrame = parentFrame;
        quit = false;

        setKeyBindings();

        fillDialog();
    }

    /** Erstellt Tastenkombinationen. */
    private void setKeyBindings() {
        setKeyBindingEscape(() -> chooseToGoON());
        setKeyBindingEnter(() -> chooseToQuit());
    }

    /** Baut die Gui auf. */
    @Override
    protected void populateDialog() {
        add(createButtonPanel(), BorderLayout.CENTER);

        if (parentFrame != null) {
            Point location = parentFrame.getLocation();
            int x = (int) location.getX();
            int y = (int) location.getY();
            setLocation(x + 265, y + 90);
        }
        else {
            setLocation(350, 300);
        }
    }

    /** Erzeugt den Bereich mit den Buttons. */
    private JPanel createButtonPanel() {
        JPanel panel = new JPanel();
        setColors(panel);
        panel.setLayout(new BorderLayout());

        JLabel innerTitle = new JLabel(
                "Möchten sie das Programm wirklich beenden?");
        innerTitle.setBorder(new EmptyBorder(5, 10, 5, 10));
        setColors(innerTitle);
        panel.add(innerTitle, BorderLayout.NORTH);

        JPanel subPanel = new JPanel();
        setColors(subPanel);
        subPanel.setLayout(new FlowLayout(FlowLayout.CENTER, 25, 10));
        panel.add(subPanel, BorderLayout.CENTER);

        JButton quitButton = new JButton("Ja");
        JButton goOnButton = new JButton("Nein");
        quitButton.setFocusable(false);
        goOnButton.setFocusable(false);
        setColors(quitButton);
        setColors(goOnButton);
        subPanel.add(quitButton);
        subPanel.add(goOnButton);
        quitButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                chooseToQuit();
            }
        });
        goOnButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                chooseToGoON();
            }
        });

        return panel;
    }

    /** Wählt als Ergebnis des Dialogs, weiter zu machen. */
    private void chooseToGoON() {
        quit = false;
        closeDialog();
    }

    /** Wählt als Ergebnis des Dialogs, das Spiel zu beenden. */
    private void chooseToQuit() {
        quit = true;
        closeDialog();
    }

    /**
     * Ermittelt, ob wirklich beendet werden soll.
     *
     * @return true: Wirklich beenden.
     */
    public boolean doQuit() {
        return quit;
    }

}
