package de.duehl.gameoflife.ui.panel;

import java.awt.Point;
import java.awt.Rectangle;

/**
 * Diese Klasse hält die Informationen über eine Zelle zur Abfrage von
 * Mauskoordinaten vor.
 *
 * @version 1.01     2013-10-30
 * @author Christian Dühl
 */

public class Cell {

    /** x-Koordinate im Spielfeld. */
    private final int x;

    /** y-Koordinate im Spielfeld. */
    private final int y;

    /** Koordinaten der Zelle auf dem Panel. */
    private final Rectangle rectangle;

    /**
     * Konstruktor
     *
     * @param x
     *            x-Koordinate im Spielfeld.
     * @param y
     *            y-Koordinate im Spielfeld.
     * @param rectangle
     *            Koordinaten der Zelle auf dem Panel.
     */
    public Cell(final int x, final int y, final Rectangle rectangle) {
        super();
        this.x = x;
        this.y = y;
        this.rectangle = rectangle;
    }

    /** Getter für die x-Koordinate im Spielfeld. */
    public int getX() {
        return x;
    }

    /** Getter für die y-Koordinate im Spielfeld. */
    public int getY() {
        return y;
    }

//    /** Getter für die Koordinaten der Zelle auf dem Panel. */
//    public Rectangle getRectangle() {
//        return rectangle;
//    }

    /**
     * Stellt fest, ob ein Punkt zu dieser Zelle gehört.
     *
     * @param point
     *            Zu überprüfender Punkt.
     * @return true: Der Punkt gehört zu dieser Zelle.
     *         false: Der Punkt gehört nicht zu dieser Zelle.
     */
    public boolean isStruckedby(final Point point) {
        return rectangle.contains(point);
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + x;
        result = prime * result + y;
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        Cell other = (Cell) obj;
        if (x != other.x) {
            return false;
        }
        if (y != other.y) {
            return false;
        }
        return true;
    }

}
