package de.duehl.gameoflife.logic.field;

import static org.junit.Assert.*;

import org.junit.Test;

import de.duehl.gameoflife.logic.field.GameField;
import de.duehl.gameoflife.logic.randomBoolean.TrueBooleanInitializer;

public class GameFieldTest {

    private final static boolean A = true;  // alive cell
    private final static boolean D = false; // dead cell

    @Test
    public void create1() {
        GameField field = new GameField(10, 10);
        assertNotNull(field);
    }

    @Test
    public void getWidth1() {
        GameField field = new GameField(20, 10);
        int width = field.getWidth();
        assertEquals(20, width);
    }

    @Test
    public void getHeight1() {
        GameField field = new GameField(20, 10);
        int height = field.getHeight();
        assertEquals(10, height);
    }

    @Test
    public void initialisedFieldIsEmpty() {
        GameField field = new GameField(5, 5);
        checkIfFieldIsEmpty(field);
    }

    @Test (expected = IllegalArgumentException.class)
    public void isCellLiving1() {
        GameField field = new GameField(3, 3);
        field.isCellAlive(3, 1);
    }

    @Test (expected = IllegalArgumentException.class)
    public void isCellLiving2() {
        GameField field = new GameField(3, 3);
        field.isCellAlive(-1, 1);
    }

    @Test (expected = IllegalArgumentException.class)
    public void isCellLiving3() {
        GameField field = new GameField(3, 3);
        field.isCellAlive(1, 3);
    }

    @Test (expected = IllegalArgumentException.class)
    public void isCellLiving4() {
        GameField field = new GameField(3, 3);
        field.isCellAlive(1, -1);
    }

    @Test
    public void clearFieldAndAllIsEmpty1() {
        GameField field = new GameField(3, 3);
        field.clear();
        checkIfFieldIsEmpty(field);
    }

    @Test (expected = IllegalArgumentException.class)
    public void setCell1() {
        GameField field = new GameField(3, 3);
        field.setCell(3, 1, true);
    }

    @Test (expected = IllegalArgumentException.class)
    public void setCell2() {
        GameField field = new GameField(3, 3);
        field.setCell(-1, 1, true);
    }

    @Test (expected = IllegalArgumentException.class)
    public void setCell3() {
        GameField field = new GameField(3, 3);
        field.setCell(1, 3, true);
    }

    @Test (expected = IllegalArgumentException.class)
    public void setCell4() {
        GameField field = new GameField(3, 3);
        field.setCell(1, -1, true);
    }

    @Test
    public void setCell5() {
        GameField field = new GameField(3, 3);
        field.setCell(0, 0, true);
        boolean living = field.isCellAlive(0, 0);
        assertEquals(true, living);
    }

    @Test (expected = IllegalArgumentException.class)
    public void create2() {
        boolean[][] initialField = {
                { D, D, D, D, D },
                { D, D, A, D, D },
                { D, A, D, A    },     // unterschiedliche Breite!
                { D, D, A, D, D },
                { D, D, D, D, D },
        };
        new GameField(initialField);
    }

    @Test (expected = IllegalArgumentException.class)
    public void create3() {
        boolean[][] initialField = { }; // Höhe 0
        new GameField(initialField);
    }

    @Test (expected = IllegalArgumentException.class)
    public void create4() {
        boolean[][] initialField = { {} }; // Breite 0
        new GameField(initialField);
    }

    @Test
    public void create5() {
        boolean[][] initialField = {
                { D, D, D, D, D },
                { D, D, A, D, D },
                { D, A, D, A, D },
                { D, D, A, D, D },
                { D, D, D, D, D },
        };
        GameField field = new GameField(initialField);
        assertNotNull(field);
    }

    @Test
    public void setInitializer1() {
        GameField field = new GameField(3, 3);
        field.setBooleanInitializer(new TrueBooleanInitializer());
    }

    @Test
    public void createRandomField1() {
        GameField field = new GameField(3, 3);
        field.setBooleanInitializer(new TrueBooleanInitializer());
        field.createRandomField();
        for (int y=0; y<3; y++) {
            for (int x=0; x<3; x++) {
                boolean living = field.isCellAlive(x, y);
                assertEquals(true, living);
            }
        }
    }

    @Test
    public void createRandomField2() {
        GameField field = new GameField(3, 3);
        field.createRandomField();
        // Ein echt zufälliges Feld kann nicht geprüft werden, aber der Aufruf
        // soll nicht fehlschlagen!
    }

    @Test
    public void clearFieldAndAllIsEmpty2() {
        GameField field = new GameField(3, 3);
        field.setBooleanInitializer(new TrueBooleanInitializer());
        field.createRandomField();
        field.clear();
        checkIfFieldIsEmpty(field);
    }

    private void checkIfFieldIsEmpty(GameField field) {
        int width = field.getWidth();
        int height = field.getHeight();
        for (int y=0; y<height; ++y) {
            for (int x=0; x<width; ++x) {
                boolean living = field.isCellAlive(x, y);
                assertEquals(false, living);
            }
        }
    }

    @Test
    public void calculateNextGeneration1() {
        boolean[][] initialField = {
                { D, D, D, D, D },
                { D, D, A, D, D },
                { D, A, D, A, D },
                { D, D, A, D, D },
                { D, D, D, D, D },
        };
        GameField field = new GameField(initialField);
        field.calculateNextGeneration();

    }

    @Test
    public void calculateNextGeneration2() {
        boolean[][] initialField = {
                { D, D, D, D, D },
                { D, D, A, D, D },
                { D, A, D, A, D },
                { D, D, A, D, D },
                { D, D, D, D, D },
        }; // konstante Figur
        GameField field = new GameField(initialField);
        field.calculateNextGeneration();
        int width = field.getWidth();
        int height = field.getHeight();
        for (int y=0; y<height; ++y) {
            for (int x=0; x<width; ++x) {
                boolean living1 = field.isCellAlive(x, y);
                boolean living2 = initialField[x][y];
                assertEquals(living1, living2);
            }
        }
    }

    @Test
    public void countLivingNeighbours1() {
        boolean[][] initialField = {
                { D, D, D },
                { D, D, D },
                { D, D, D },
        };
        GameField field = new GameField(initialField);
        int countLivingNeighbours1 = field.countLivingNeighbours(0, 0);
        int countLivingNeighbours2 = field.countLivingNeighbours(0, 1);
        int countLivingNeighbours3 = field.countLivingNeighbours(0, 2);
        int countLivingNeighbours4 = field.countLivingNeighbours(1, 0);
        int countLivingNeighbours5 = field.countLivingNeighbours(1, 1);
        int countLivingNeighbours6 = field.countLivingNeighbours(1, 2);
        int countLivingNeighbours7 = field.countLivingNeighbours(2, 0);
        int countLivingNeighbours8 = field.countLivingNeighbours(2, 1);
        int countLivingNeighbours9 = field.countLivingNeighbours(2, 2);
        assertEquals(0, countLivingNeighbours1);
        assertEquals(0, countLivingNeighbours2);
        assertEquals(0, countLivingNeighbours3);
        assertEquals(0, countLivingNeighbours4);
        assertEquals(0, countLivingNeighbours5);
        assertEquals(0, countLivingNeighbours6);
        assertEquals(0, countLivingNeighbours7);
        assertEquals(0, countLivingNeighbours8);
        assertEquals(0, countLivingNeighbours9);
    }

    @Test
    public void countLivingNeighbours2() {
        boolean[][] initialField = {
                { D, A, D },
                { A, D, A },
                { D, A, D },
        };
        GameField field = new GameField(initialField);
        int countLivingNeighbours1 = field.countLivingNeighbours(0, 0);
        int countLivingNeighbours2 = field.countLivingNeighbours(0, 1);
        int countLivingNeighbours3 = field.countLivingNeighbours(0, 2);
        int countLivingNeighbours4 = field.countLivingNeighbours(1, 0);
        int countLivingNeighbours5 = field.countLivingNeighbours(1, 1);
        int countLivingNeighbours6 = field.countLivingNeighbours(1, 2);
        int countLivingNeighbours7 = field.countLivingNeighbours(2, 0);
        int countLivingNeighbours8 = field.countLivingNeighbours(2, 1);
        int countLivingNeighbours9 = field.countLivingNeighbours(2, 2);
        assertEquals(2, countLivingNeighbours1);
        assertEquals(2, countLivingNeighbours2);
        assertEquals(2, countLivingNeighbours3);
        assertEquals(2, countLivingNeighbours4);
        assertEquals(4, countLivingNeighbours5);
        assertEquals(2, countLivingNeighbours6);
        assertEquals(2, countLivingNeighbours7);
        assertEquals(2, countLivingNeighbours8);
        assertEquals(2, countLivingNeighbours9);
    }

    @Test
    public void countLivingNeighbours3() {
        boolean[][] initialField = {
                { A, A, A },
                { A, D, A },
                { A, A, A },
        };
        GameField field = new GameField(initialField);
        int countLivingNeighbours1 = field.countLivingNeighbours(0, 0);
        int countLivingNeighbours2 = field.countLivingNeighbours(0, 1);
        int countLivingNeighbours3 = field.countLivingNeighbours(0, 2);
        int countLivingNeighbours4 = field.countLivingNeighbours(1, 0);
        int countLivingNeighbours5 = field.countLivingNeighbours(1, 1);
        int countLivingNeighbours6 = field.countLivingNeighbours(1, 2);
        int countLivingNeighbours7 = field.countLivingNeighbours(2, 0);
        int countLivingNeighbours8 = field.countLivingNeighbours(2, 1);
        int countLivingNeighbours9 = field.countLivingNeighbours(2, 2);
        assertEquals(2, countLivingNeighbours1);
        assertEquals(4, countLivingNeighbours2);
        assertEquals(2, countLivingNeighbours3);
        assertEquals(4, countLivingNeighbours4);
        assertEquals(8, countLivingNeighbours5);
        assertEquals(4, countLivingNeighbours6);
        assertEquals(2, countLivingNeighbours7);
        assertEquals(4, countLivingNeighbours8);
        assertEquals(2, countLivingNeighbours9);
    }

    @Test
    public void calculateNextGeneration3() {
        boolean[][] initialField = {
                { D, D, D, D, D },
                { D, D, D, D, D },
                { D, A, A, A, D },
                { D, D, D, D, D },
                { D, D, D, D, D },
        }; // Blinker: - | - | - ...
        GameField field = new GameField(initialField);
        //System.out.println(field + "\n");
        field.calculateNextGeneration();
        //System.out.println(field + "\n");
        boolean[][] initialFieldNextStep = {
                { D, D, D, D, D },
                { D, D, A, D, D },
                { D, D, A, D, D },
                { D, D, A, D, D },
                { D, D, D, D, D },
        }; // Blinker: | - | - | ...
        int width = field.getWidth();
        int height = field.getHeight();
        for (int y=0; y<height; ++y) {
            //System.out.println("\ny = " + y);
            for (int x=0; x<width; ++x) {
                //System.out.println("x = " + x);
                boolean living1 = field.isCellAlive(x, y);
                //System.out.println("ist : " + living1);
                boolean living2 = initialFieldNextStep[x][y];
                //System.out.println("soll: " + living2);
                assertEquals(living1, living2);
            }
        }
    }

    @Test
    public void getWidth2() {
        boolean[][] initialField = {
                { D, D, D, D, D },
                { D, D, D, D, D },
                { D, A, A, A, D },
                { D, D, D, D, D },
                { D, D, D, D, D },
        };
        GameField field = new GameField(initialField);
        int width = field.getWidth();
        assertEquals(5, width);
    }

    @Test
    public void getHeight2() {
        boolean[][] initialField = {
                { D, D, D, D, D },
                { D, D, D, D, D },
                { D, A, A, A, D },
                { D, D, D, D, D },
                { D, D, D, D, D },
        };
        GameField field = new GameField(initialField);
        int height = field.getHeight();
        assertEquals(5, height);
    }

    @Test
    public void clearFieldAndAllIsEmpty3() {
        boolean[][] initialField = {
                { D, D, D, D, D },
                { D, D, D, D, D },
                { D, D, D, D, D },
                { D, D, D, D, D },
                { D, D, D, D, D },
        };
        GameField field = new GameField(initialField);
        checkIfFieldIsEmpty(field);
    }

    @Test (expected = IllegalArgumentException.class)
    public void insertWrongSubField1() {
        GameField field = new GameField(20, 20);
        boolean[][] subField = {
                { D, D, D, D, D },
                { D, D, A, D, D },
                { D, A, D, A    },     // unterschiedliche Breite!
                { D, D, A, D, D },
                { D, D, D, D, D },
        };
        field.insertSubField(subField, 0, 0);
    }

    @Test (expected = IllegalArgumentException.class)
    public void insertWrongSubField2() {
        GameField field = new GameField(20, 20);
        boolean[][] subField = { }; // Höhe 0
        field.insertSubField(subField, 0, 0);
    }

    @Test (expected = IllegalArgumentException.class)
    public void insertWrongSubField3() {
        GameField field = new GameField(20, 20);
        boolean[][] subField = { {} }; // Breite 0
        field.insertSubField(subField, 0, 0);
    }

    @Test (expected = IllegalArgumentException.class)
    public void insertRightSubFieldAtWrongPlace1() {
        GameField field = new GameField(20, 20);
        boolean[][] subField = {
                { A, A, A },
                { A, A, A },
                { A, A, A },
        };
        field.insertSubField(subField, -1, 0);
    }

    @Test (expected = IllegalArgumentException.class)
    public void insertRightSubFieldAtWrongPlace2() {
        GameField field = new GameField(20, 20);
        boolean[][] subField = {
                { A, A, A },
                { A, A, A },
                { A, A, A },
        };
        field.insertSubField(subField, 0, -1);
    }

    @Test (expected = IllegalArgumentException.class)
    public void insertRightSubFieldAtWrongPlace3() {
        GameField field = new GameField(20, 20);
        boolean[][] subField = {
                { A, A, A },
                { A, A, A },
                { A, A, A },
        };
        field.insertSubField(subField, 20, 0);
    }

    @Test (expected = IllegalArgumentException.class)
    public void insertRightSubFieldAtWrongPlace4() {
        GameField field = new GameField(20, 20);
        boolean[][] subField = {
                { A, A, A },
                { A, A, A },
                { A, A, A },
        };
        field.insertSubField(subField, 0, 20);
    }

    @Test (expected = IllegalArgumentException.class)
    public void insertRightSubFieldAtWrongPlace5() {
        GameField field = new GameField(20, 20);
        boolean[][] subField = {
                { A, A, A },
                { A, A, A },
                { A, A, A },
        };
        field.insertSubField(subField, 0, 18);
    }

    @Test (expected = IllegalArgumentException.class)
    public void insertRightSubFieldAtWrongPlace6() {
        GameField field = new GameField(20, 20);
        boolean[][] subField = {
                { A, A, A },
                { A, A, A },
                { A, A, A },
        };
        field.insertSubField(subField, 18, 0);
    }

    @Test (expected = IllegalArgumentException.class)
    public void insertRightSubFieldAtWrongPlace7() {
        GameField field = new GameField(20, 20);
        boolean[][] subField = {
                { A, A, A },
                { A, A, A },
                { A, A, A },
        };
        field.insertSubField(subField, 18, 18);
    }

    @Test
    public void insertSubField1() {
        GameField field = new GameField(5, 5);
        boolean[][] subField = {
                { A, A, A },
                { A, A, A },
                { A, A, A },
        };
        field.insertSubField(subField, 0, 0);

        assertEquals(true, field.isCellAlive(0, 0));
        assertEquals(true, field.isCellAlive(0, 1));
        assertEquals(true, field.isCellAlive(0, 2));
        assertEquals(true, field.isCellAlive(1, 0));
        assertEquals(true, field.isCellAlive(1, 1));
        assertEquals(true, field.isCellAlive(1, 2));
        assertEquals(true, field.isCellAlive(2, 0));
        assertEquals(true, field.isCellAlive(2, 1));
        assertEquals(true, field.isCellAlive(2, 2));
    }

    @Test
    public void insertSubField2() {
        GameField field = new GameField(5, 5);
        boolean[][] subField = {
                { A, A, A },
                { A, A, A },
                { A, A, A },
        };
        field.insertSubField(subField, 1, 0);

        assertEquals(true, field.isCellAlive(1, 0));
        assertEquals(true, field.isCellAlive(1, 1));
        assertEquals(true, field.isCellAlive(1, 2));
        assertEquals(true, field.isCellAlive(2, 0));
        assertEquals(true, field.isCellAlive(2, 1));
        assertEquals(true, field.isCellAlive(2, 2));
        assertEquals(true, field.isCellAlive(3, 0));
        assertEquals(true, field.isCellAlive(3, 1));
        assertEquals(true, field.isCellAlive(3, 2));
    }

    @Test
    public void insertSubField3() {
        GameField field = new GameField(5, 5);
        boolean[][] subField = {
                { A, A, A },
                { A, A, A },
                { A, A, A },
        };
        field.insertSubField(subField, 2, 0);

        assertEquals(true, field.isCellAlive(2, 0));
        assertEquals(true, field.isCellAlive(2, 1));
        assertEquals(true, field.isCellAlive(2, 2));
        assertEquals(true, field.isCellAlive(3, 0));
        assertEquals(true, field.isCellAlive(3, 1));
        assertEquals(true, field.isCellAlive(3, 2));
        assertEquals(true, field.isCellAlive(4, 0));
        assertEquals(true, field.isCellAlive(4, 1));
        assertEquals(true, field.isCellAlive(4, 2));
    }

    @Test
    public void insertSubField4() {
        GameField field = new GameField(5, 5);
        boolean[][] subField = {
                { A, A, A },
                { A, A, A },
                { A, A, A },
        };
        field.insertSubField(subField, 2, 2);

        assertEquals(true, field.isCellAlive(2, 2));
        assertEquals(true, field.isCellAlive(2, 3));
        assertEquals(true, field.isCellAlive(2, 4));
        assertEquals(true, field.isCellAlive(3, 2));
        assertEquals(true, field.isCellAlive(3, 3));
        assertEquals(true, field.isCellAlive(3, 4));
        assertEquals(true, field.isCellAlive(4, 2));
        assertEquals(true, field.isCellAlive(4, 3));
        assertEquals(true, field.isCellAlive(4, 4));
    }

    @Test
    public void insertSubField5() {
        GameField field = new GameField(5, 5);
        boolean[][] subField = {
                { D, A, D },
                { A, D, A },
                { D, A, D },
        };
        field.insertSubField(subField, 1, 1);

        assertEquals(false, field.isCellAlive(1, 1));
        assertEquals(true , field.isCellAlive(1, 2));
        assertEquals(false, field.isCellAlive(1, 3));
        assertEquals(true , field.isCellAlive(2, 1));
        assertEquals(false, field.isCellAlive(2, 2));
        assertEquals(true , field.isCellAlive(2, 3));
        assertEquals(false, field.isCellAlive(3, 1));
        assertEquals(true , field.isCellAlive(3, 2));
        assertEquals(false, field.isCellAlive(3, 3));
    }

    @Test
    public void toggle1() {
        GameField field = new GameField(5, 5);
        field.toggle(0, 0);
        assertEquals(true, field.isCellAlive(0, 0));
    }

    @Test
    public void setCellLiving1() {
        GameField field = new GameField(5, 5);
        field.setCellLiving(0, 0, true);
        assertEquals(true, field.isCellAlive(0, 0));
    }

    @Test
    public void setCellLiving2() {
        GameField field = new GameField(5, 5);
        field.setCellLiving(0, 0, true);
        field.setCellLiving(0, 0, false);
        assertEquals(false, field.isCellAlive(0, 0));
    }

}
