package de.duehl.basics.datetime.date;

/*
 * Copyright 2017 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.util.Calendar;

/**
 * Eine Hilfsklasse rund um das Datum.
 *
 * @version 1.01     2017-03-28
 * @author Christian Dühl
 */

public class DateHelper {

    /** Erzeugt das heutige Datum. */
    public static ImmutualDate actualDate() {
        Calendar cal = Calendar.getInstance();
        return calendarToDate(cal); // Der normale Konstruktor täte es auch ... new ImmutualDate()
    }

    /** Gibt das heutige Datum im Format "TT.MM.YYYY" zurück. */
    public static String actualDateAsString() {
        Calendar cal = Calendar.getInstance();
        ImmutualDate iDate = calendarToDate(cal);
        return iDate.toString();
    }

    /** Erzeugt das Datum aus einem Calendar-Objekt. */
    public static ImmutualDate calendarToDate(Calendar cal) {
        int day = cal.get(Calendar.DAY_OF_MONTH);
        int month = cal.get(Calendar.MONTH) + 1;
        int year = cal.get(Calendar.YEAR);

        return new ImmutualDate(day, month, year);
    }

    /** Stellt fest, ob der angegebene Wochentag heute ist. */
    public static boolean weekDayIsToday(Weekday weekday) {
        ImmutualDate today = new ImmutualDate();
        Weekday weekdayToday = today.dayOfTheWeek();
        return weekdayToday == weekday;
    }

}
