package de.duehl.basics.io;

/*
 * Copyright 2016 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

/**
 * Enum für die Charsets.
 *
 * @see java.nio.charset.Charset
 *
 * @version 1.03     2017-12-19
 * @author Christian Dühl
 */

public enum Charset {

    US_ASCII(
            "US-ASCII",
            "Seven-bit ASCII, a.k.a. ISO646-US, a.k.a. the Basic Latin block of the Unicode "
                    + "character set"),
    ISO_8859_1(
            "ISO-8859-1",
            "ISO Latin Alphabet No. 1, a.k.a. ISO-LATIN-1"),
    UTF_8(
            "UTF-8",
            "Eight-bit UCS Transformation Format"),
    UTF_16BE(
            "UTF-16BE",
            "Sixteen-bit UCS Transformation Format, big-endian byte order"),
    UTF_16LE(
            "UTF-16LE",
            "Sixteen-bit UCS Transformation Format, little-endian byte order"),
    UTF_16(
            "UTF-16",
            "Sixteen-bit UCS Transformation Format, byte order identified by an optional "
                    + "byte-order mark"),
    ;

    // "Cp1252" ?

    private final String charsetAsString;
    private final String description;

    private Charset(String charsetAsString, String description) {
        this.charsetAsString = charsetAsString;
        this.description = description;
    }

    public String getCharsetAsString() {
        return charsetAsString;
    }

    public String getDescription() {
        return description;
    }

    public static Charset getDefaultCharset() {
        return ISO_8859_1;
    }

}
