package de.duehl.basics.text;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import static org.junit.Assert.*;

import java.util.List;

import org.junit.Test;

public class NumberStringTest {

    @Test
    public void taupu01() {
        int zahl = 1;
        String expected = "1";
        String actual = NumberString.taupu(zahl);
        assertEquals(expected, actual);
    }

    @Test
    public void taupu02() {
        int zahl = 999;
        String expected = "999";
        String actual = NumberString.taupu(zahl);
        assertEquals(expected, actual);
    }

    @Test
    public void taupu03() {
        int zahl = 1000;
        String expected = "1.000";
        String actual = NumberString.taupu(zahl);
        assertEquals(expected, actual);
    }

    @Test
    public void taupu04() {
        int zahl = 999999;
        String expected = "999.999";
        String actual = NumberString.taupu(zahl);
        assertEquals(expected, actual);
    }

    @Test
    public void taupu05() {
        int zahl = 1234567;
        String expected = "1.234.567";
        String actual = NumberString.taupu(zahl);
        assertEquals(expected, actual);
    }

    @Test
    public void taupu80k() {
        int zahl = 80_000;
        String expected = "80.000";
        String actual = NumberString.taupu(zahl);
        assertEquals(expected, actual);
    }

    @Test
    public void taupu05long() {
        long zahl = 1234567;
        String expected = "1.234.567";
        String actual = NumberString.taupu(zahl);
        assertEquals(expected, actual);
    }

    @Test
    public void removeTaupuEmptyInput() {
        String taupu = "";
        String actual = NumberString.removeTaupu(taupu);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void removeTaupuNoPoint() {
        String taupu = "123";
        String actual = NumberString.removeTaupu(taupu);
        String expected = "123";
        assertEquals(expected, actual);
    }

    @Test
    public void removeTaupuOnePoint() {
        String taupu = "999.999";
        String actual = NumberString.removeTaupu(taupu);
        String expected = "999999";
        assertEquals(expected, actual);
    }

    @Test
    public void removeTaupuTwoPoints() {
        String taupu = "1.234.567";
        String actual = NumberString.removeTaupu(taupu);
        String expected = "1234567";
        assertEquals(expected, actual);
    }

    @Test
    public void isDigitSequenceEmptyString() {
        assertFalse(NumberString.isDigitSequence(""));
    }

    @Test
    public void isDigitSequenceSpaceBefore() {
        assertFalse(NumberString.isDigitSequence(" 123"));
    }

    @Test
    public void isDigitSequenceSpaceBehind() {
        assertFalse(NumberString.isDigitSequence("123 "));
    }

    @Test
    public void isDigitSequenceSpaceBeforeAndBehind() {
        assertFalse(NumberString.isDigitSequence(" 123 "));
    }

    @Test
    public void isDigitPerhapsWithPointSequence() {
        assertTrue(NumberString.isDigitPerhapsWithPointSequence("123.0"));
    }

    @Test
    public void isDigitPerhapsWithPointSequenceWithoutPoint() {
        assertTrue(NumberString.isDigitPerhapsWithPointSequence("123"));
    }

    @Test
    public void isDigitPerhapsWithPointSequenceOnlyPointIsWrong() {
        assertFalse(NumberString.isDigitPerhapsWithPointSequence("."));
    }

    @Test
    public void isDigitPerhapsWithPointSequenceStartingWithPointIsWrong() {
        assertFalse(NumberString.isDigitPerhapsWithPointSequence(".0123"));
    }

    @Test
    public void isDigitPerhapsWithCommaSequence() {
        assertTrue(NumberString.isDigitPerhapsWithCommaSequence("123,0"));
    }

    @Test
    public void isDigitPerhapsWithCommaSequenceWithoutPoint() {
        assertTrue(NumberString.isDigitPerhapsWithCommaSequence("123"));
    }

    @Test
    public void isDigitPerhapsWithCommaSequenceOnlyCommaIsWrong() {
        assertFalse(NumberString.isDigitPerhapsWithCommaSequence(","));
    }

    @Test
    public void isDigitPerhapsWithCommaSequenceCommaIsWrong() {
        assertFalse(NumberString.isDigitPerhapsWithCommaSequence(",0123"));
    }

    @Test
    public void isDigitSequence01() {
        assertTrue(NumberString.isDigitSequence("012345"));
    }

    @Test
    public void isDigitSequenceWithDot() {
        assertFalse(NumberString.isDigitSequence("123.0"));
    }

    @Test
    public void isDigitSequenceWithCommy() {
        assertFalse(NumberString.isDigitSequence("123,0"));
    }

    @Test
    public void removeLeadingZeroesEmptyString() {
        String input = "";
        String actual = NumberString.removeLeadingZeroes(input);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLeadingZeroesOnlyZeroes() {
        String input = "0000000";
        String actual = NumberString.removeLeadingZeroes(input);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void removeLeadingZeroesSpaceBeforeZeroes() {
        String input = " 0000000";
        String actual = NumberString.removeLeadingZeroes(input);
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void removeLeadingZeroesNormalCase() {
        String input = "000123";
        String actual = NumberString.removeLeadingZeroes(input);
        String expected = "123";
        assertEquals(expected, actual);
    }

    @Test
    public void removeZeroEndingEmptyString() {
        String input = "";
        String actual = NumberString.removeZeroEnding(input);
        String expected = "";
        assertEquals(expected, actual);
    }

    @Test
    public void removeZeroEndingWithZeroWithoutDotAtEnd() {
        String input = "20";
        String actual = NumberString.removeZeroEnding(input);
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void removeZeroEndingWithNoNumberAtEnd() {
        String input = "12345.000 xxx";
        String actual = NumberString.removeZeroEnding(input);
        String expected = input;
        assertEquals(expected, actual);
    }

    @Test
    public void removeZeroEndingLeadingZeroesAndDot() {
        String input = "12345.000";
        String actual = NumberString.removeZeroEnding(input);
        String expected = "12345";
        assertEquals(expected, actual);
    }

    @Test
    public void removeZeroEndingLeadingDot() {
        String input = "12345.";
        String actual = NumberString.removeZeroEnding(input);
        String expected = "12345";
        assertEquals(expected, actual);
    }

    @Test
    public void removeZeroEndingLeadingZeros() {
        String input = "12345.12000";
        String actual = NumberString.removeZeroEnding(input);
        String expected = "12345.12";
        assertEquals(expected, actual);
    }

    @Test
    public void cleanNumberEmptyNumber() {
        String text = "";
        String expected = "";
        String actual = NumberString.cleanNumber(text);
        assertEquals(expected, actual);
    }

    @Test
    public void cleanNumber01() {
        String text = "10.234.642,12";
        String expected = "10234642.12";
        String actual = NumberString.cleanNumber(text);
        assertEquals(expected, actual);
    }

    @Test
    public void cleanNumber02() {
        String text = "10234642.12";
        String expected = "10234642.12";
        String actual = NumberString.cleanNumber(text);
        assertEquals(expected, actual);
    }

    @Test
    public void cleanNumberTwoCOmmas() {
        String text = "102,34642,12";
        String expected = "1023464212";
        String actual = NumberString.cleanNumber(text);
        assertEquals(expected, actual);
    }

    @Test
    public void findNumbersWithoutNumber() {
        String in = "Müll";
        List<Integer> numbers = NumberString.findNumbers(in);
        assertTrue(numbers.isEmpty());
    }

    @Test
    public void findNumbersWithPureNumber() {
        String in = "12345";
        List<Integer> numbers = NumberString.findNumbers(in);
        assertEquals(1, numbers.size());
        assertEquals(Integer.valueOf(12345), numbers.get(0));
    }

    @Test
    public void findNumbersWithOneNumber() {
        String in = "foo12345bar";
        List<Integer> numbers = NumberString.findNumbers(in);
        assertEquals(1, numbers.size());
        assertEquals(Integer.valueOf(12345), numbers.get(0));
    }

    @Test
    public void findNumbersWithOneNumberWithSpace() {
        String in = "foo 12345 bar";
        List<Integer> numbers = NumberString.findNumbers(in);
        assertEquals(1, numbers.size());
        assertEquals(Integer.valueOf(12345), numbers.get(0));
    }

    @Test
    public void findNumbersWithTwoNumbers() {
        String in = "100,32";
        List<Integer> numbers = NumberString.findNumbers(in);
        assertEquals(2, numbers.size());
        assertEquals(Integer.valueOf(100), numbers.get(0));
        assertEquals(Integer.valueOf( 32), numbers.get(1));
    }

    @Test
    public void findNumbersWithThreeNumbers() {
        String in = "100.32E002";
        List<Integer> numbers = NumberString.findNumbers(in);
        assertEquals(3, numbers.size());
        assertEquals(Integer.valueOf(100), numbers.get(0));
        assertEquals(Integer.valueOf( 32), numbers.get(1));
        assertEquals(Integer.valueOf(  2), numbers.get(2));
    }

    @Test
    public void addLeadingZeroesWithNegativeNumber() {
        int input = -1;
        int numberOfDigits = 13;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "-1";
        assertEquals(expected, actual);
    }

    @Test
    public void addLeadingZeroesWithZero() {
        int input = 0;
        int numberOfDigits = 3;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "000";
        assertEquals(expected, actual);
    }

    @Test
    public void addLeadingZeroesWithLongerNumber() {
        int input = 123;
        int numberOfDigits = 2;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "123";
        assertEquals(expected, actual);
    }

    @Test
    public void addLeadingZeroesWithExaktLongNumber() {
        int input = 123;
        int numberOfDigits = 3;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "123";
        assertEquals(expected, actual);
    }

    @Test
    public void addLeadingZeroesWithShorterNumber() {
        int input = 12;
        int numberOfDigits = 3;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "012";
        assertEquals(expected, actual);
    }

    @Test
    public void addLeadingZeroesLongWithNegativeNumber() {
        long input = -1;
        int numberOfDigits = 13;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "-1";
        assertEquals(expected, actual);
    }

    @Test
    public void addLeadingZeroesLongWithZero() {
        long input = 0;
        int numberOfDigits = 3;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "000";
        assertEquals(expected, actual);
    }

    @Test
    public void addLeadingZeroesLongWithLongerNumber() {
        long input = 123;
        int numberOfDigits = 2;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "123";
        assertEquals(expected, actual);
    }

    @Test
    public void addLeadingZeroesLongWithExaktLongNumber() {
        long input = 123;
        int numberOfDigits = 3;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "123";
        assertEquals(expected, actual);
    }

    @Test
    public void addLeadingZeroesLongWithShorterNumber() {
        long input = 12;
        int numberOfDigits = 3;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "012";
        assertEquals(expected, actual);
    }

    @Test
    public void addLeadingZeroesStringWithNegativeNumber() {
        String input = "-1";
        int numberOfDigits = 13;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "-1";
        assertEquals(expected, actual);
    }

    @Test
    public void addLeadingZeroesStringWithZero() {
        String input = "0";
        int numberOfDigits = 3;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "000";
        assertEquals(expected, actual);
    }

    @Test
    public void addLeadingZeroesStringWithStringerNumber() {
        String input = "123";
        int numberOfDigits = 2;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "123";
        assertEquals(expected, actual);
    }

    @Test
    public void addLeadingZeroesStringWithExaktStringNumber() {
        String input = "123";
        int numberOfDigits = 3;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "123";
        assertEquals(expected, actual);
    }

    @Test
    public void addLeadingZeroesStringWithShorterNumber() {
        String input = "12";
        int numberOfDigits = 3;
        String actual = NumberString.addLeadingZeroes(input, numberOfDigits);
        String expected = "012";
        assertEquals(expected, actual);
    }

    @Test
    public void parseIntGood() {
        String in = "15";
        int actual = NumberString.parseInt(in, "Hier sollte kein Fehler auftreten!");
        int expected = 15;
        assertEquals(expected, actual);
    }

    @Test (expected = RuntimeException.class)
    public void parseIntBadNumber() {
        String in = "15,7";
        NumberString.parseInt(in, "Zahl mit Komma kann nicht geparst werden!");
    }

    @Test (expected = RuntimeException.class)
    public void parseIntBadDot() {
        String in = "15.7";
        NumberString.parseInt(in, "Zahl mit Punkt kann nicht geparst werden!");
    }

    @Test (expected = RuntimeException.class)
    public void parseIntBad() {
        String in = "Müll";
        NumberString.parseInt(in, "Buchstaben können nicht geparst werden!");
    }

    @Test
    public void parseIntIgnoreGood() {
        String in = "15";
        int actual = NumberString.parseIntIgnore(in, -1);
        int expected = 15;
        assertEquals(expected, actual);
    }

    @Test
    public void parseIntIgnoreBadNumber() {
        String in = "15,7";
        int actual = NumberString.parseIntIgnore(in, -1);
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void parseIntIgnoreBadDot() {
        String in = "15.7";
        int actual = NumberString.parseIntIgnore(in, -1);
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void parseIntIgnoreBad() {
        String in = "Müll";
        int actual = NumberString.parseIntIgnore(in, -1);
        int expected = -1;
        assertEquals(expected, actual);
    }

    @Test
    public void isInteger0() {
        String number = "0";
        assertTrue(NumberString.isInteger(number));
    }

    @Test
    public void isIntegerMinusOne() {
        String number = "-1";
        assertTrue(NumberString.isInteger(number));
    }

    @Test
    public void isIntegerEmptyString() {
        String number = "";
        assertFalse(NumberString.isInteger(number));
    }

    @Test
    public void isIntegerWhitespace() {
        String number = " ";
        assertFalse(NumberString.isInteger(number));
    }

    @Test
    public void isIntegerNineDigitNumber() {
        String number = "800112471";
        assertTrue(NumberString.isInteger(number));
    }

    @Test
    public void isIntegerHighestNineDigitNumber() {
        String number = "999999999";
        assertTrue(NumberString.isInteger(number));
    }

    @Test
    public void isIntegerBigNumber() {
        String number = "8001124714237260";
        assertFalse(NumberString.isInteger(number));
    }

    @Test
    public void parseLongGood() {
        String in = "8001124714237260";
        long actual = NumberString.parseLong(in, "Hier sollte kein Fehler auftreten!");
        long expected = 8001124714237260L;
        assertEquals(expected, actual);
    }

    @Test (expected = RuntimeException.class)
    public void parseLongBadNumber() {
        String in = "15,7";
        NumberString.parseLong(in, "Zahl mit Komma kann nicht geparst werden!");
    }

    @Test
    public void parseDoubleGood() {
        String in = "15.7";
        double actual = NumberString.parseDouble(in, "Hier sollte kein Fehler auftreten!");
        double expected = 15.7d;
        assertEquals(expected, actual, 0.0001);
    }

    @Test (expected = RuntimeException.class)
    public void parseDoubleBadNumber() {
        String in = "fünfzehn komma sieben!";
        NumberString.parseDouble(in, "Text kann nicht geparst werden!");
    }

    @Test
    public void twoDigitStringToInteger15() {
        String in = "15";
        int actual = NumberString.twoDigitStringToInteger(in);
        int expected = 15;
        assertEquals(expected, actual);
    }

    @Test
    public void twoDigitStringToInteger01() {
        String in = "01";
        int actual = NumberString.twoDigitStringToInteger(in);
        int expected = 1;
        assertEquals(expected, actual);
    }

    @Test (expected = RuntimeException.class)
    public void twoDigitStringToIntegerTooShort() {
        String in = "1";
        NumberString.twoDigitStringToInteger(in);
    }

    @Test (expected = RuntimeException.class)
    public void twoDigitStringToIntegerTooLong() {
        String in = "123";
        NumberString.twoDigitStringToInteger(in);
    }

    @Test (expected = RuntimeException.class)
    public void twoDigitStringToIntegerNoNumber() {
        String in = "ab";
        NumberString.twoDigitStringToInteger(in);
    }

    @Test (expected = RuntimeException.class)
    public void twoDigitStringToIntegerNoNumberAtSecondPlace() {
        String in = "0a";
        NumberString.twoDigitStringToInteger(in);
    }

    @Test
    public void twoDecimalPlaces() {
        double input = 12345.67890123d;
        String actual = NumberString.twoDecimalPlaces(input);
        String expected = "12345,68";
        assertEquals(expected, actual);
    }

    @Test
    public void twoDecimalPlacesTaupu() {
        double input = 12345.67890123d;
        String actual = NumberString.twoDecimalPlacesTaupu(input);
        String expected = "12.345,68";
        assertEquals(expected, actual);
    }

    @Test
    public void twoDecimalPlacesTaupuTwoDecimals() {
        double input = 12345.67d;
        String actual = NumberString.twoDecimalPlacesTaupu(input);
        String expected = "12.345,67";
        assertEquals(expected, actual);
    }

    @Test
    public void twoDecimalPlacesTaupuOneDecimal() {
        double input = 12345.6d;
        String actual = NumberString.twoDecimalPlacesTaupu(input);
        String expected = "12.345,60";
        assertEquals(expected, actual);
    }

    @Test
    public void twoDecimalPlacesTaupuWithoutDecimals() {
        double input = 12345d;
        String actual = NumberString.twoDecimalPlacesTaupu(input);
        String expected = "12.345,00";
        assertEquals(expected, actual);
    }

    @Test
    public void twoDecimalPlacesTaupuBig() {
        double input = 12345e12;
        String actual = NumberString.twoDecimalPlacesTaupu(input);
        String expected = "12.345.000.000.000.000,00";
        assertEquals(expected, actual);
    }

    @Test
    public void calculateDigits0() {
        int number = 0;
        int actual = NumberString.calculateDigits(number);
        int expected = 1;
        assertEquals(expected, actual);
    }

    @Test (expected = RuntimeException.class)
    public void calculateDigitsMinusOne() {
        int number = -1;
        NumberString.calculateDigits(number);
    }

    @Test
    public void calculateDigits1() {
        int number = 1;
        int actual = NumberString.calculateDigits(number);
        int expected = 1;
        assertEquals(expected, actual);
    }

    @Test
    public void calculateDigits9() {
        int number = 9;
        int actual = NumberString.calculateDigits(number);
        int expected = 1;
        assertEquals(expected, actual);
    }

    @Test
    public void calculateDigits10() {
        int number = 10;
        int actual = NumberString.calculateDigits(number);
        int expected = 2;
        assertEquals(expected, actual);
    }

    @Test
    public void calculateDigits99() {
        int number = 99;
        int actual = NumberString.calculateDigits(number);
        int expected = 2;
        assertEquals(expected, actual);
    }

    @Test
    public void calculateDigits100() {
        int number = 100;
        int actual = NumberString.calculateDigits(number);
        int expected = 3;
        assertEquals(expected, actual);
    }

    @Test
    public void calculateDigits999() {
        int number = 999;
        int actual = NumberString.calculateDigits(number);
        int expected = 3;
        assertEquals(expected, actual);
    }

    @Test
    public void calculateDigits1000() {
        int number = 1000;
        int actual = NumberString.calculateDigits(number);
        int expected = 4;
        assertEquals(expected, actual);
    }

    @Test
    public void calculateDigits9999() {
        int number = 9999;
        int actual = NumberString.calculateDigits(number);
        int expected = 4;
        assertEquals(expected, actual);
    }

    @Test
    public void percent2From10() {
        int count = 2;
        int total = 10;
        String actual = NumberString.percent(count, total);
        String expected = "20,00";
        assertEquals(expected, actual);
    }

    @Test
    public void percent20From100() {
        int count = 20;
        int total = 100;
        String actual = NumberString.percent(count, total);
        String expected = "20,00";
        assertEquals(expected, actual);
    }

    @Test
    public void percent0From80() {
        int count = 0;
        int total = 80;
        String actual = NumberString.percent(count, total);
        String expected = "0,00";
        assertEquals(expected, actual);
    }

    @Test
    public void percent12From12() {
        int count = 12;
        int total = 12;
        String actual = NumberString.percent(count, total);
        String expected = "100,00";
        assertEquals(expected, actual);
    }

    @Test
    public void percent0From0() {
        int count = 0;
        int total = 0;
        String actual = NumberString.percent(count, total);
        String expected = "NaN";
        assertEquals(expected, actual);
    }

    @Test
    public void toGermanNumberZero() {
        String actual = NumberString.toGermanNumber("0");
        String expected = "Null";
        assertEquals(expected, actual);
    }

    @Test
    public void toGermanNumberOne() {
        String actual = NumberString.toGermanNumber("1");
        String expected = "Ein";
        assertEquals(expected, actual);
    }

    @Test
    public void toGermanNumberOther() {
        String actual = NumberString.toGermanNumber("999");
        String expected = "999";
        assertEquals(expected, actual);
    }

    @Test
    public void germanPluralZero() {
        String actual = NumberString.germanPlural(0, "Worte", "Wort");
        String expected = "Worte";
        assertEquals(expected, actual);
    }

    @Test
    public void germanPluralOne() {
        String actual = NumberString.germanPlural(1, "Worte", "Wort");
        String expected = "Wort";
        assertEquals(expected, actual);
    }

    @Test
    public void germanPluralOther() {
        String actual = NumberString.germanPlural(999, "Worte", "Wort");
        String expected = "Worte";
        assertEquals(expected, actual);
    }

}
