package de.duehl.swing.ui.buttons.painted;

/*
 * Copyright 2017 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import de.duehl.swing.ui.PixelPoint;

/**
 * Diese Klasse stellt einen kleinen Schalter zum Zurückspulen von etwas dar.
 *
 * @version 1.01     2017-07-04
 * @author Christian Dühl
 */

public class GoBackButton extends PaintedButton {

    private static final long serialVersionUID = -1441507523499749679L;

    /** Konstruktor. */
    public GoBackButton() {
        super();
    }

    /** Zeichnet das Symbol in horizontaler und rechteckiger Anordnung. */
    @Override
    protected void paintHorizontalSymbolRectangle() {
        paintSymbol();
    }

    /** Zeichnet das Symbol in vertikaler und rechteckiger Anordnung. */
    @Override
    protected void paintVerticalSymbolRectangle() {
        paintSymbol();
    }

    /** Zeichnet das Symbol in horizontaler und quadratischer Anordnung. */
    @Override
    protected void paintHorizontalSymbolSquare() {
        paintSymbol();
    }

    /** Zeichnet das Symbol in vertikaler und quadratischer Anordnung. */
    @Override
    protected void paintVerticalSymbolSquare() {
        paintSymbol();
    }

    /** Zeichnet den kleinen Pfeil zum Zurückspulen. */
    private void paintSymbol() {
        int width = getWidth();
        int height = getHeight();

        if (isForceSquare()) {
            width = Math.min(width, height);
            height = width;
        }

        int delta = 3;

        int firstLeftPixel = (getWidth() - width)/2;
        int left = firstLeftPixel + delta;

        int lastRightPixel = firstLeftPixel + width - 1;
        int right = lastRightPixel - delta;

        int firstUpperPixel = (getHeight() - height)/2;
        int upper = firstUpperPixel + delta;

        int lastLowerPixel = firstUpperPixel + height - 1;
        int lower = lastLowerPixel - delta;

        int midHeight = firstUpperPixel + height / 2;
        int midWidth = firstLeftPixel + width / 2;

        if (isHorizontal()) {
            paintPolygon(
                    new PixelPoint(right,  upper    ),
                    new PixelPoint(left,   midHeight),
                    new PixelPoint(right,  lower    )
                    );
        }
        else {
            paintPolygon(
                    new PixelPoint(left,     lower),
                    new PixelPoint(midWidth, upper),
                    new PixelPoint(right,    lower)
                    );
        }
    }

}